/****************************************************
* COS265 Data Structures & Algorithms               *
* Project 5 - Small Group kD Trees                  *
* by Chad Jordan 11/16/07                           *
*                                                   *
* Java implementation of a basic visualizer         *
* for BST point search using the swing framework.   *
* Dependencies: none                                *
****************************************************/
import javax.swing.*;
import java.awt.*;
import java.awt.geom.Point2D;

public class Visualizer extends JPanel {

    private RedBlackPointSearch redBlackPointSearch;
    private Point2D.Double queryPoint;
    private Point2D.Double nearestPoint;

    public Visualizer(RedBlackPointSearch redBlackPointSearch) {
        this.redBlackPointSearch = redBlackPointSearch;
        setPreferredSize(new Dimension(800, 600));
        queryPoint = new Point2D.Double(0, 0);
        nearestPoint = null;
    }

    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2d = (Graphics2D) g;

        // Draw points
        g2d.setColor(Color.BLUE);
        for (double x = 0; x <= 10; x += 0.5) {
            for (double y = 0; y <= 10; y += 0.5) {
                Point2D.Double point = new Point2D.Double(x, y);
                g2d.fillOval((int) (point.getX() * 50), (int) ((10 - point.getY()) * 50), 5, 5);
            }
        }

        // Draw query point
        g2d.setColor(Color.RED);
        g2d.fillOval((int) (queryPoint.getX() * 50), (int) ((10 - queryPoint.getY()) * 50), 10, 10);

        // Draw nearest point
        if (nearestPoint != null) {
            g2d.setColor(Color.GREEN);
            g2d.fillOval((int) (nearestPoint.getX() * 50), (int) ((10 - nearestPoint.getY()) * 50), 10, 10);

            // Draw line between query point and nearest point
            g2d.setColor(Color.BLACK);
            g2d.drawLine((int) (queryPoint.getX() * 50) + 5, (int) ((10 - queryPoint.getY()) * 50) + 5,
                    (int) (nearestPoint.getX() * 50) + 5, (int) ((10 - nearestPoint.getY()) * 50) + 5);
        }
    }

    public void setQueryPoint(Point2D.Double queryPoint) {
        this.queryPoint = queryPoint;
        nearestPoint = redBlackPointSearch.nearest(queryPoint);
        repaint();
    }

    public static void main(String[] args) {
        RedBlackPointSearch redBlackPointSearch = new RedBlackPointSearch();
        redBlackPointSearch.insert(new Point2D.Double(2, 3));
        redBlackPointSearch.insert(new Point2D.Double(5, 4));
        redBlackPointSearch.insert(new Point2D.Double(9, 6));
        redBlackPointSearch.insert(new Point2D.Double(4, 7));

        Visualizer visualizer = new Visualizer(redBlackPointSearch);

        JFrame frame = new JFrame("Point Search Visualizer");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.getContentPane().add(visualizer);
        frame.pack();
        frame.setLocationRelativeTo(null);
        frame.setVisible(true);

        // Set query point and visualize
        Point2D.Double queryPoint = new Point2D.Double(6, 5);
        visualizer.setQueryPoint(queryPoint);
    }
}